using System;
using System.Transactions;
using System.Data.SqlClient;

namespace transactie
{
    class Program
    {
        public static void Main(String[] args)
        {
            Console.WriteLine("Bedrag op rekening 1 : " + Rekening.GetBedrag(1));
            Console.WriteLine("Bedrag op rekening 2 : " + Rekening.GetBedrag(2));
            try
            {
                //de laatste parameter is type decimal, vandaar "m"
                Rekening.SchrijfGeldOver(1, 2, 150.0m);
                Console.WriteLine("Overschrijving gelukt");
            }
            catch (Exception ex)
            {
                Console.WriteLine("Er is een fout opgetreden:" + ex.Message);
            }
            Console.WriteLine("Bedrag op rekening 1 : " + Rekening.GetBedrag(1));
            Console.WriteLine("Bedrag op rekening 2 : " + Rekening.GetBedrag(2));

        }
    }
    class Rekening
    {
        private const String CONNECTIONSTRING = @"Data Source=(localDB)\V11.0;AttachDbFilename=|DataDirectory|\introdotnet.mdf;Integrated Security=True";

        public static decimal GetBedrag(int reknr)
        {
            using (SqlConnection conn = new SqlConnection(CONNECTIONSTRING))
            {
                SqlCommand commando = new SqlCommand("Select bedrag from Rekening where nr = @nr",conn);
                commando.Parameters.AddWithValue("@nr", reknr);
                conn.Open();
                return (decimal)commando.ExecuteScalar();
            }
        }
        public static void SchrijfGeldOver(int van, int naar, decimal bedrag)
        {
            //via TransactionScope kunnen we verschillende statements in dezelfde transactie zetten
            TransactionScope ts = new TransactionScope();
            try
            {
                //"using" genereert dezelfde code als try..finally in C#:
                // SqlConnection conn = new SqlConnection(CONNECTIONSTRING);
                //try{
                //
                //}finally{
                //  conn.Close();
                //}
                using (SqlConnection conn = new SqlConnection(CONNECTIONSTRING))
                {
                    //twee SQL statements met parameters (@bedrag, @nr)
                    SqlCommand command1 = new SqlCommand("update Rekening set bedrag = bedrag - @bedrag where nr=@nr", conn);
                    SqlCommand command2 = new SqlCommand("update Rekening set bedrag = bedrag + @bedrag where nr=@nr", conn);
                    //die hier worden ingevuld
                    command1.Parameters.AddWithValue("@bedrag", bedrag);
                    command1.Parameters.AddWithValue("@nr", van);
                    command2.Parameters.AddWithValue("@bedrag", bedrag);
                    command2.Parameters.AddWithValue("@nr", naar);
                    conn.Open();
                    //voer non-select statements uit
                    //de returnwaarde van ExecuteNonQuery geeft het aantal records terug dat is aangepast
                    if (command2.ExecuteNonQuery() != 1) throw new Exception("Rekening " + naar + " bestaat niet");
                    if (command1.ExecuteNonQuery() != 1) throw new Exception("Rekening " + van + " bestaat niet");
                }
                //Als er geen exception is opgetreden => Complete() oproepen
                ts.Complete();
            }
            finally
            {
                //altijd Dispose oproepen
                ts.Dispose();
            }
        }
    }
}
